package de.uni_passau.fim;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.GradientDrawable;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RadioGroup;
import android.widget.RadioGroup.OnCheckedChangeListener;
import de.uni_passau.fim.gui.ColorRadioButton;

/**
 * This class displays the game activity. The layout of the
 * game is provided in the file <code>res/layout/main.xml</code>.
 */
public class MasterMind extends Activity {

	//*************************************************************************
		// Global variables
		//*************************************************************************
		
		
		//*************************************************************************
		/* These numbers represent the parameters of the game.
		 * They are assigned dynamically in the onCreate(Bundle) below. */
		
		/** Number of colors the ColorSelector offers */
		private int numberOfColors;
		
		/** This is the number of possible tries. */
		private int numberOfRows;
		
		/** The number of colors that make one guess. */
		private int numberOfPlaces;
		
		//*************************************************************************
		/* The following variables represent the current state of the game. */
		
		/** The quest is the color sequence which has to be found by the player. */
		private int[] quest;
		
		/** This is the color that is currently selected by the ColorSelector. */
		private int currentColor; 
		
		/** The row of the board that has to be filled with the guess+answer.  */
		private int currentRow;
		
		/** The guess contains the currently set colors of the ColorSetter for easy access. */
		private int guess[]; // TODO: Aufgabe 2b) und 4a)
		
		/** The colors of the ColorSelector are collected in this array. */
		private int[] colors;
		
		/* These arrays could provide an internal representation of the game, similar to the guess array above.
		 * This is useful to compute the hint. */ 
		// private int[][] guesses;
		
		// private int[][] answers;
		
		/* Use these colors to set the answer fields. */
		
		/** Color of an empty place in the answers. */
		private final int colorUnused = 0xff000000;
		
		/** Color of an place in the answers. */
		private final int colorExact = 0xffff0000;
		
		/** Color of an empty place in the answers. */
		private final int colorCorrect = 0xffffffff;
		
		
		/** The ID of the 'you lost'-dialog */
		private final int DIALOG_GAMEOVER_ID = 0;
		
		/** The ID of the 'you win'-dialog */
		private final int DIALOG_CORRECT_ID = 1;
		
		
		/** This map tells us if the given ImageView (= circle of the ColorSetter)
		 * is the first, second, ...
		 * Access to a Map is provided by the methods put(...) and get(...). */
		private Map<ImageView, Integer> colorSetterMap;
		
		/** This array provides an easy access to the (n+1)-th row of the 'Board' in 
		 * 'main.xml'. It is filled in onCreate(Bundle) below.  */
		private ViewGroup[] boardRowMap;

		/** Static reference to the multiplayer class if
		   we are in multiplayer mode. */
		private static Multiplayer mpClass;

		
		//*************************************************************************
		// Public getter/setter
		//*************************************************************************
		
		/**
		 * Getter for mpClass
		 * @return Multiplayer
		 */
		public static Multiplayer getMpClass() {
			return mpClass;
		}

		/**
		 * Setter for mpClass
		 * @param mpClass the partent MPClass
		 */
		public static void setMpClass(Multiplayer mpClass) {
			MasterMind.mpClass = mpClass;
		}
	
	//*************************************************************************
	// LISTENERS
	//*************************************************************************
	
	/** This listener is assigned to the ColorSelector. It is called if a new
	 * color is selected and it updates the 'currentColor'. */
	private OnCheckedChangeListener colorSelectorListener = new OnCheckedChangeListener() {

		public void onCheckedChanged(RadioGroup group, int checkedId) {
			View v = findViewById(checkedId);
			if (v instanceof ColorRadioButton) {
				currentColor = ((ColorRadioButton) v).getColor();
			}
		}
	};
	
	/** This listener is assigned to the fields of the 'ColorSetter'.
	 * On click it assigns the currently selected color 'currentColor'
	 * to the field. */
	private OnClickListener colorSetterListener = new OnClickListener() {

		public void onClick(View v) {

			if (v instanceof ImageView) {
				// TODO: Aufgabe 2 b): Assign the color of currentColor to the ImageView.
				//       Hint: You can use the helper method 'setColor'
			}
			// TODO: Aufgabe 4 a): validate if we can enable the check button
		}
	};

	
	/*  
	 * TODO Aufgabe 2a) Add this listener to the "Check"-button, see below in method onCreate(Bundle). 
	 */
	/** The listener for the "Check"-button. */
	private OnClickListener checkButtonListener = new OnClickListener() {
		public void onClick(View button) {
			updateBoard(/* TODO: your parameters - (Aufgabe 3, see also below in section "game logic")  */);
		}
	};
	
	/** The listener for the "Hint"-button. */
	private OnClickListener hintButtonListener = new OnClickListener() {
		public void onClick(View button) {
			computeAndSetHint();
		}
	};


	
	//*************************************************************************
	// onCreate methods (Activity, Dialogs)
	//*************************************************************************

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		
		currentColor = Color.WHITE;

		
		// First collect the colors provided by 'ColorSelector'
		// to the 'colors' array.
		RadioGroup rg = (RadioGroup) this.findViewById(R.id.ColorSelector);
		rg.setOnCheckedChangeListener(colorSelectorListener);
		numberOfColors = rg.getChildCount();
		colors = new int[numberOfColors];
		for (int i = 0; i < rg.getChildCount(); i++) {
			View v = rg.getChildAt(i);
			if (v instanceof ColorRadioButton) {
				colors[i] = ((ColorRadioButton) v).getColor();
			}
		}
		
		// Now handle the 'ColorSetter'. This gives us the number of places.
		ViewGroup current = (ViewGroup) this.findViewById(R.id.ColorSetter);
		colorSetterMap = new HashMap<ImageView, Integer>();
		int placesCounter = 0;
		for (int i = 0; i < current.getChildCount(); i++) {
			View v = current.getChildAt(i);
			if (v instanceof ImageView) {
				v.setOnClickListener(colorSetterListener);
				colorSetterMap.put((ImageView) v, i);
				placesCounter++;
			}
		}
		numberOfPlaces = placesCounter;
		
		// Handle the 'Board' and get the 'numberOfRows'.
		ViewGroup board = (ViewGroup) findViewById(R.id.Board);
		numberOfRows = board.getChildCount();
		boardRowMap = new ViewGroup[numberOfRows];
		for (int j = 0; j < numberOfRows; j++) {
			View v = board.getChildAt(j);
			if (v instanceof ViewGroup) {
				boardRowMap[j] = (ViewGroup) v;
			}
		}
		
		setUpQuest();
		
		currentRow = 0;

	    /* TODO: Aufgabe 2a:
	     *       Assign listener 'checkButtonListener' to the 'check'-Button 
	     *       Do the same for the 'hint' button. */

	}
	
	/**
	 * Creates a dialoag based on the given id
	 * 
	 * @param id the dialoag ID
	 * @return the dialog
	 */
	protected Dialog onCreateDialog(int id) {
		Dialog dialog = null;
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		switch (id) {
		case DIALOG_CORRECT_ID:
			// TODO: Aufgabe 4 b): Create an 'you win' dialog!
			break;
		case DIALOG_GAMEOVER_ID:
			// TODO: Aufgabe 4 b): This dialog doesn't look nice!
			builder.setMessage(R.string.gameover)
					.setPositiveButton(R.string.OK,
							new DialogInterface.OnClickListener() {
								public void onClick(DialogInterface dialog,
										int id) {
									MasterMind.this.finish();
								}
							});
			dialog = builder.create();
			break;
		default:
			dialog = null;
		}
		return dialog;
	}
	
	
	//*************************************************************************
	// Game Logic
	//*************************************************************************

	/**
	 * Checks, if the current guess is correct.
	 * If true, it displays the "You win"-dialog. If not the
	 * board displays the new guess and the result. If the 
	 * last row is used, the user loses the game.
	 * 
	 * @param
	 */
	private void updateBoard(/* perhaps you need some parameters */) {
	    /* TODO: Aufgabe 3
	     * 
	     * a) Get the current colors from R.id.ColorSetter.
	     * b) Check the current guess and compute the answer.
	     * c) Set a new row on the game board (with hints for the player).
	     * d) Check if the game is over.
	     */
	}

	/* TODO Strategie-Aufgaben: Compute a hint and set it up in the ColorSetter. */
	private void computeAndSetHint() {
		
	}
	
	
	//*************************************************************************
	// Helper methods.
	//*************************************************************************

	/**
	 * Assign a color to a image view, i.e. an element of a row.
	 * 
	 * @param image on this view to color is assigned 
	 * @param color the assigned color
	 */
	protected static void setColor(ImageView image, int color) {
		Drawable d = image.getDrawable();
		if (d instanceof GradientDrawable) {
			GradientDrawable gd = (GradientDrawable) image.getDrawable();
			gd.mutate();
			gd.setColor(color);
			gd.invalidateSelf();
		}
	}
	
	/**
	 * Randomly fill the quest array with colors available in the 'colors' array. 
	 * 
	 */
	private void setUpQuest() {
		quest = new int[numberOfPlaces];
		int[] readableQuest = new int[numberOfPlaces];
		Random r = new Random();
		for(int i = 0; i < quest.length; i++) {
			int k = r.nextInt(colors.length);
			quest[i] = colors[k];
			readableQuest[i] = k;
			System.out.println("Setting color " + k);
		}
		System.out.println("The quest (readbl): " + Arrays.toString(readableQuest));
		System.out.println("The quest (colors): " + Arrays.toString(quest));
	}
	
}