package de.uni_passau.fim.strategies;

import java.util.Arrays;

public class CompleteGuessIterator implements IGuessIterator {

	int mNumOfColors;
	int mNumOfPlaces;
	boolean mWithRepetition;
	
	int[] mNextGuess;
	int[] colorCounter;
	boolean mHasNext;
	
	int mSize;
	int mCurrPos;
	
	public CompleteGuessIterator(int numOfColors, int numOfPlaces, boolean withRepetition) {
		
		mNumOfColors = numOfColors;
		mNumOfPlaces = numOfPlaces;
		mWithRepetition = withRepetition;
		colorCounter = new int[mNumOfColors];
		if (withRepetition) {
			mHasNext = true;
			mNextGuess = new int[mNumOfPlaces]; 
			Arrays.fill(mNextGuess, 0);
			colorCounter[0] = mNumOfPlaces;
		} else if (!withRepetition && mNumOfColors < mNumOfPlaces) {
			mHasNext = false;
			return;
		} else {
			mHasNext = true;
			mNextGuess = new int[mNumOfPlaces];
			for (int i=0; i<mNumOfPlaces; i++) {
				mNextGuess[i] = i;
				colorCounter[i]++;
			}
		}
		mCurrPos = 0;
		mSize = computeSize();
	}

	public boolean hasNext() {
		return mHasNext;
	}
	
	public Guess next() {
		if (!hasNext())
			return null;
		int[] current = mNextGuess.clone();
		if (mWithRepetition)
			computeNextWithRepetition();
		else
			computeNextWithoutRepetition();
		return new Guess(current, mNumOfColors);
	}

	private void computeNextWithoutRepetition() {
		boolean overflow = increment();
		
		while (!overflow && hasRepetition())
			overflow = increment();
		
		if (overflow)
			mHasNext = false;
	}
	
	private boolean hasRepetition() {
		for (int i=0; i<colorCounter.length; i++)
			if (colorCounter[i] > 1)
				return true;
		return false;
	}

	private void computeNextWithRepetition() {
		boolean overflow = increment();
		if (overflow)
			mHasNext = false;
	}
	
	
	private boolean increment() {
		int pos = mNumOfPlaces-1;
		boolean hasOverflow = incrementPlace(pos);
		
		while (hasOverflow && --pos >= 0) {
			hasOverflow = incrementPlace(pos);
			if (pos == 0 && hasOverflow)
				return true;
		}
		return false;
	}
	
	private boolean incrementPlace(int i) {
		int tmp = mNextGuess[i];
		colorCounter[tmp]--;
		tmp = (++tmp)%mNumOfColors;
		mNextGuess[i] = tmp;
		colorCounter[tmp]++;
		return tmp==0;
	}
	
	private int computeSize() {
		if (mWithRepetition)
			return (int)Math.pow(mNumOfColors, mNumOfPlaces);
		else {
			int tmp = mNumOfColors;
			for (int i=1; i<mNumOfPlaces; i++ ) {
				tmp *= (mNumOfColors-i);
			}
			return tmp;
		}
	}

	public int size() {
		return mSize;
	}

	public int currPos() {
		return mCurrPos;
	}
	
}
