package de.uni_passau.fim.strategies;

/**
 * Class to provide a very simple logging mechanism: For instance, in
 * <code>debug("Hello");</code>, "Hello" is only printed to standard out
 * if the log level is set to DEBUG, but not, if the level is set to VERBOSE,
 * INFO or SILENT. In <code>info("Hello");</code>, "Hello" is printed for all
 * levels except SILENT.
 */
public class Logger {

	public static final int DEBUG = 0;
	public static final int VERBOSE = 1;
	public static final int INFO = 2;
	public static final int SILENT = Integer.MAX_VALUE;
	
	public static int level = 2;
	
	public static void log(String s, int level) {
		if (level >= Logger.level)
			System.out.println(s);
	}
	
	/**
	 * Prints string s only if the log level is debug or above.
	 * @param s
	 */
	public static void debug(String s) {
		Logger.log(s, Logger.DEBUG);
	}
	
	/**
	 * Prints string s only if the log level is verbose or above. 
	 * @param s
	 */
	public static void verbose(String s) {
		Logger.log(s, Logger.VERBOSE);
	}
	
	/**
	 * Prints string s only if the log level is info or above. 
	 * @param s
	 */
	public static void info(String s) {
		Logger.log(s, Logger.INFO);
	}
	
	/**
	 * Sets the debug level to l.
	 * @param l
	 */
	public static void setLevel(int l) {
		level = l;
	}
	
}
